Scriptname GLorthiem extends ReferenceAlias

Actor lorthiem
GEmotionCombinator es
GEmotionDecay ed
GSecondaryAppraisal sa
GBehaviourRegulation br
GExpressiveCommunication ec
GSensoryInput si
GPrimaryAppraisal pa
GAttention a

Actor Property test Auto
Actor Property PlayerREF Auto

;;GPlayer playerData
float lastPlayerProfile = 0.0

;; float normalizedMaximum = Math.sqrt(3)
float scale = 57.7
int previousExpressedEmotion = -1
float normalEnergyLevel

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Event: OnInit
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This event is native to the Creation Kit library. Since this script will not be available while the event is active, it only gathers and 
;; records script references attached to the NPC. It registers for an update event to finish initialization so that it can register other 
;; events.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Event OnInit()	
	Initialization()
	return
EndEvent

Function Initialization()
		lorthiem = ((self as ReferenceAlias).GetReference() as Actor)
	si = (self as Alias) as GSensoryInput
	a = (self as Alias) as GAttention
	pa = (self as Alias) as GPrimaryAppraisal
	es = (self as Alias) as GEmotionCombinator
	ed = (self as Alias) as GEmotionDecay
	sa = (self as Alias) as GSecondaryAppraisal
	ec = (self as Alias) as GExpressiveCommunication
	br = (self as Alias) as GBehaviourRegulation
	si.AssignProfileObject(((self as Alias)) as GProfile)	
	
	;;playerData = (self.GetOwningQuest().GetAliasByName("PlayerData")) as GPlayer
	
	RegisterForSingleUpdate(1.0)
	
	return
EndFunction

Function TestLortheim()
	Debug.Notification(lorthiem.GetBaseObject().GetName() + " is active")
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Event: OnUpdate (Empty State)
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This event is registered for in the OnInit() event to finish the initialization of required values, including the equilibrium emotions in 
;; GEmotionState, the threshold values in GSecondaryApprisal, and profile creation in GSensoryInput.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Event OnUpdate()		
	;; Retrieve the equilibrium emotional state and update the emotion store
	es.SetEquilibriumEmotions(ed.GetEquilibriumJoy(), 100 - ed.GetEquilibriumJoy(), ed.GetEquilibriumTrust(), 100 - ed.GetEquilibriumTrust(), ed.GetEquilibriumFear(), 100 - ed.GetEquilibriumFear(), ed.GetEquilibriumAnticipation(), 100 - ed.GetEquilibriumAnticipation())
	
	;; Make sure that the number of threshold values equals the number of emotion labels
	if(!sa.CheckThresholdValidity())
		Debug.MessageBox("Invalid number of emotion threshold values for: " + lorthiem.GetBaseObject().GetName())
	endif
	
	;; Even if the appropriate values cannot be calculated at first, this function needs to be 
	;; called so that the profile object has a reference to the associated Actor object
	if (!si.isProcessingProfile())
		si.SelfAnalyse(lorthiem)
	endif
	
	normalEnergyLevel = lorthiem.GetActorValue("Energy")
	
	;;Debug.MessageBox("Empty State OnUpdate")
	;;RegisterForSingleLOSGain(lorthiem, PlayerREF)
	
	GotoState("Ready") ;; Go to a state with a different OnUpdate() definition
	RegisterForSingleUpdate(1.0)
	
	return
EndEvent

;;Event OnGainLOS(Actor akViewer, ObjectReference akTarget)
	
	;;if (akTarget == PlayerREF)
		;;Debug.Notification("Lortheim sees you")
		;;RunGLaDOSForPlayer(playerData.GetSelfProfile())	
	;;endif
	
	;;return
;;EndEvent

State Ready

	;; ---------------------------------------------------------------------------------------------------------------------------------------------
	;; Event: OnUpdate (Ready State)
	;; ---------------------------------------------------------------------------------------------------------------------------------------------
	;; This event is registered to perform regular calls to GEmotionDecay.
	;; ---------------------------------------------------------------------------------------------------------------------------------------------
	Event OnUpdate()
		;;Debug.Notification("Ready State OnUpdate")
		
		;; -------------------------------------
		;; Decay emotions
		;; -------------------------------------
		;;Debug.MessageBox("Emotions post-decay: " + es.GetValues())
		int[] decay = ed.DecayEmotions(es.GetJoy(), es.GetSadness(), es.GetTrust(), es.GetDisgust(), es.GetAnger(), es.GetFear(), es.GetAnticipation(), es.GetSurprise())
		es.UpdateEmotions(decay[0], decay[1], decay[2], decay[3], decay[4], decay[5], decay[6], decay[7])
		;;Debug.MessageBox("[" + lorthiem.GetBaseObject().GetName() + "] Emotions post-decay: " + es.GetValues())
		
		;; -------------------------------------
		;; Check secondary appraisal to see if a change in behaviour is required
		;; -------------------------------------
		int emState = sa.DetermineEmotionState(CompileEmotionIntensities())
		;;Debug.MessageBox(CompileEmotionIntensities() + "\nDecaying index to: " + emState)
		
		;; -------------------------------------
		;; Change flags in the behaviour regulation and expressive communication blocks
		;; -------------------------------------
		UpdateBehaviours(emState)
		
		RegisterForSingleUpdate(ed.GetDecayRate())
		
		return
	EndEvent

EndState

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: RunGLaDOSForPlayer
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function passes a GPlayerProfile object through the GLaDOS architecture.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Function RunGLaDOSForPlayer (GPlayerProfile playPro)
	bool process = false

	;; -------------------------------------
	;; Check the last time we got the player's profile and the last time the profile was updated
	;; -------------------------------------
	if (playPro.GetProfileCreationTime() > lastPlayerProfile)
		process = true
		lastPlayerProfile = playPro.GetProfileCreationTime()
	endif	
	
	;;Debug.Notification("[GLorthiem (" + lorthiem.GetBaseObject().GetName() + ")] Process flag = " + process + "(Player profile timestamp = " + playPro.GetProfileCreationTime())
	;; -------------------------------------
	;; We need to process this profile object
	;; -------------------------------------
	if(process)
		;;Debug.Notification("[GLorthiem (" + lorthiem.GetBaseObject().GetName() + ")] In the processing block")
		lorthiem.SetLookAt(playPro.GetProfileOwner(), false)
		
		;; -------------------------------------
		;; Get the stimuli and dedications from the player profile and pass to GAttention to see if the data should be processed
		;; -------------------------------------
		int[] stimIndex = new int[3]
		stimIndex[0] = a.isImportantStimuli(playPro.GetProfileOwnerRace(), 1.0)
		;;Debug.MessageBox("Stimuli " + stimIndex[0] + " = " + playPro.GetProfileOwnerRace())
		
		string[] stim = playPro.GetStimuli()
		float[] dedications = playPro.GetStimuliDedication()
		int i = 1
		int j = 0
		while (i < 3)
			stimIndex[i] = -1
			while (j < stim.Length)
				stimIndex[i] = a.isImportantStimuli(stim[j], dedications[j])
				if (stimIndex[i] > -1)
					;;Debug.MessageBox("Stimuli = " + stim[j])
					j = stim.Length ;; break						
				endif
				j = j + 1
			endwhile
			i = i + 1
		endwhile
		;;Debug.MessageBox("Indicies found: " + stimIndex)
		
		;; -------------------------------------
		;; Get any items that the player has that could affect this NPC's appraisal
		;; -------------------------------------
		float[] items = a.GetItemInformation(playPro.GetItemNames())
		;;Debug.MessageBox("Player items: " + items)
		
		float[] goalInfo
		float[] utilities = new float [3]
		float[] responsibilities = new float [3] 
		float[] congruence = new float [3]
		float[] familiarity = new float [3]
		i = 0
		while ( i < 3 )
			;; -------------------------------------
			;; We found a stimulus that the NPC deems important -- extract all related information from GAttention (GGoals)
			;; -------------------------------------
			if (stimIndex[i] > -1)
				goalInfo = a.GetGoalInformationByStimuliIndex( stimIndex[i] )
				;;Debug.MessageBox("This goal's info: " + goalInfo)
				
				utilities[i] = goalInfo[0]
				congruence[i] = goalInfo[2]
				responsibilities[i] = goalInfo[1]
				familiarity[i] = goalInfo[3]
			endif
			i = i + 1
		endwhile
		
		;;Debug.MessageBox("Threat Control calculated: " + si.CalculatePlayerThreatControl(playerData.GetSelfProfile()))
		
		;; -------------------------------------
		;;Debug.Notification("Starting primary appraisal...")
		;; -------------------------------------
		float[] ems = pa.MultiAppraise( utilities, congruence, responsibilities, familiarity, si.CalculatePlayerThreatControl(playPro), items )
		;;Debug.MessageBox("[" + lorthiem.GetBaseObject().GetName() + "]Calculated Emotion: = " + ems[0] + "\nIntensity = " + ems[1])
		
		;; -------------------------------------
		;; Scale the calculated intensity since it has been normalized.
		;; -------------------------------------
		int scaledEmotion = Math.Ceiling( scale * ems[1] )
		
		;; -------------------------------------
		;; Send the emotional changes to the emotion store
		;; -------------------------------------
		;;Debug.MessageBox("Emotion values pre-update = " + es.GetValues())
		int code = ( ems[0] as int )
		if (code == 0)
			es.UpdateEmotions(scaledEmotion, 0, 0, 0, 0, 0, 0, 0)
		elseif (code == 1)
			es.UpdateEmotions(0, scaledEmotion, 0, 0, 0, 0, 0, 0)
		elseif (code == 2)
			es.UpdateEmotions(0, 0, scaledEmotion, 0, 0, 0, 0, 0)
		elseif (code == 3)
			es.UpdateEmotions(0, 0, 0, scaledEmotion, 0, 0, 0, 0)
		elseif (code == 4)
			es.UpdateEmotions(0, 0, 0, 0, 0, scaledEmotion, 0, 0)
		elseif (code == 5)
			es.UpdateEmotions(0, 0, 0, 0, scaledEmotion, 0, 0, 0)
		elseif (code == 6)
			es.UpdateEmotions(0, 0, 0, 0, 0, 0, scaledEmotion, 0)
		elseif (code == 7)
			es.UpdateEmotions(0, 0, 0, 0, 0, 0, 0, scaledEmotion)
		else
			Debug.Notification("Unknown emotion code encountered")
		endif
		;;Debug.MessageBox("Emotion values post-update = " + es.GetValues())
		
		;; -------------------------------------
		;; Check secondary appraisal to see if a change in behaviour is required
		;; -------------------------------------
		int emState = sa.DetermineEmotionState(CompileEmotionIntensities())
		;;Debug.MessageBox("[" + lorthiem.GetBaseObject().GetName() + "] Emotion to be expressed = " + emState)
		
		;; -------------------------------------
		;; Change flags in the behaviour regulation and expressive communication blocks
		;; -------------------------------------
		UpdateBehaviours(emState)
		
		lorthiem.ClearLookAt()
	endif
	
	return
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: UpdateBehaviours
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in an emotion code and makes the appropriate calls to GBehaviourRegulation and GExpressiveCommunication.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
Function UpdateBehaviours (int emIndex)
	int default

	bool playAnim = false
	if (previousExpressedEmotion != emIndex)
		playAnim = true
		previousExpressedEmotion = emIndex
	endif

	if (emIndex == 0)
		br.SetEmotions (true, false, false, false, false, false, false, false)
		ec.ExpressJoy(lorthiem, es.GetJoy(), playAnim)
		br.SetActorEnergy(lorthiem, es.GetJoy() as float)
		
	elseif (emIndex == 1)
		br.SetEmotions (false, true, false, false, false, false, false, false)
		ec.ExpressSadness(lorthiem, es.GetSadness(), playAnim)
		br.SetActorEnergy(lorthiem, es.GetJoy() as float)
		
	elseif (emIndex == 2)
		br.SetEmotions (false, false, true, false, false, false, false, false)
		ec.ExpressTrust(lorthiem, es.GetTrust(), playAnim)
		br.SetActorEnergy(lorthiem, normalEnergyLevel)
		
	elseif (emIndex == 3)
		br.SetEmotions (false, false, false, true, false, false, false, false)
		ec.ExpressDisgust(lorthiem, es.GetDisgust(), playAnim)
		br.SetActorEnergy(lorthiem, normalEnergyLevel)
		
	elseif (emIndex == 4)
		br.SetEmotions (false, false, false, false, true, false, false, false)
		ec.ExpressAnger(lorthiem, es.GetAnger(), playAnim)
		br.SetActorEnergy(lorthiem, es.GetAnger())
		
	elseif (emIndex == 5)
		br.SetEmotions (false, false, false, false, false, true, false, false)
		ec.ExpressFear(lorthiem, es.GetFear(), playAnim)
		br.SetActorEnergy(lorthiem, es.GetFear())
		
	elseif (emIndex == 6)
		br.SetEmotions (false, false, false, false, false, false, true, false)
		ec.ExpressAcceptance(lorthiem, es.GetAnticipation(), playAnim)
		br.SetActorEnergy(lorthiem, normalEnergyLevel)
		
	elseif (emIndex == 7)
		br.SetEmotions (false, false, false, false, false, false, false, true)
		ec.ExpressSurprise(lorthiem, es.GetSurprise(), playAnim)
		br.SetActorEnergy(lorthiem, es.GetSurprise())
	
	else
		br.SetEmotions(false, false, false, false, false, false, false, false)
		ec.ClearExpressedEmotion(lorthiem)
		
		default = sa.GetDefaultEmotion()
		if (default == 0)
			ec.ExpressJoy(lorthiem, es.GetJoy(), playAnim)
			br.SetActorEnergy(lorthiem, es.GetJoy() as float)
		elseif (default == 1)
			ec.ExpressSadness(lorthiem, es.GetSadness(), playAnim)
			br.SetActorEnergy(lorthiem, es.GetJoy() as float)
		elseif (default == 2)
			ec.ExpressTrust(lorthiem, es.GetTrust(), playAnim)
			br.SetActorEnergy(lorthiem, normalEnergyLevel)
		elseif (default == 3)
			ec.ExpressDisgust(lorthiem, es.GetDisgust(), playAnim)
			br.SetActorEnergy(lorthiem, normalEnergyLevel)
		elseif (default == 4)
			ec.ExpressAnger(lorthiem, es.GetAnger(), playAnim)
			br.SetActorEnergy(lorthiem, es.GetAnger())
		elseif (default == 5)
			ec.ExpressFear(lorthiem, es.GetFear(), playAnim)
			br.SetActorEnergy(lorthiem, es.GetFear())
		elseif (default == 6)
			ec.ExpressAcceptance(lorthiem, es.GetAnticipation(), playAnim)
			br.SetActorEnergy(lorthiem, normalEnergyLevel)
		elseif (default == 7)
			ec.ExpressSurprise(lorthiem, es.GetSurprise(), playAnim)
			br.SetActorEnergy(lorthiem, es.GetSurprise())
		endif		
	endif
	
	
	
	;; -------------------------------------
	;; Tell the NPC to re-evaluate their AI packages
	;; -------------------------------------
	if (playAnim) ;; We had an emotion change
		lorthiem.EvaluatePackage()
	endif
	
	return
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: CompileEmotionIntensities
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; A helper function that migrates all values in the emotion store to a single integer array.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int[] Function CompileEmotionIntensities()
	int[] ems = new int [8]
	ems[0] = es.GetJoy()
	ems[1] = es.GetSadness()
	ems[2] = es.GetTrust()
	ems[3] = es.GetDisgust()
	ems[4] = es.GetAnger()
	ems[5] = es.GetFear()
	ems[6] = es.GetAnticipation()
	ems[7] = es.GetSurprise()
	
	return ems
EndFunction