Scriptname GSecondaryAppraisal extends Alias

string[] Property thresholdsNames Auto
int[] Property thresholdsValues Auto
int Property defaultSelection Auto
int numEmotions

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: AssignThresholds
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in the emotion thresholds of an NPC in priority order (eg. threshName[0] takes precedence over threshName[1], etc.).
;; If the passed arrays are not the same size, this function fails (returns false).
;; ---------------------------------------------------------------------------------------------------------------------------------------------
bool Function CheckThresholdValidity ()
	bool success = false
	
	if (thresholdsNames.Length == thresholdsValues.Length)
		numEmotions = thresholdsNames.Length		
		success = true
	endif
	
	return success
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: CheckEmotionPriority
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function returns the list of threshold names, which the designer is tasked with putting in priority order for this NPC.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
string[] Function CheckEmotionPriority()
	return thresholdsNames
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: GetDefaultEmotion
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function returns the NPC's default emotional state as an encoded value.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function GetDefaultEmotion()
	return EncodeSelection(defaultSelection)
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: DetermineEmotionState
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function is used to determine which emotion an NPC should be expressing based on their current emotion values and their tolerance for
;; each emotion. The values for emValues should be passed in emotion code order (e.g. joy (0), sadness (1), ... , surprise (7)). 
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function DetermineEmotionState (int[] emValues)

	int selectedIndex = -1
	int emTemp = -1
	int i = 0	
	int lookUp
	while (i < numEmotions)	
		
		;; Determine which emValue to examine
		lookUp = EncodeSelection(i)
		
		;; The emotion value exceeds the expression threshold
		if (emValues[lookUp] >= thresholdsValues[i])			
		
			;; We haven't picked an emotion yet
			if (selectedIndex < 0)			
				selectedIndex = i
				emTemp = lookUp
				
			;; A lower priority emotion is stronger	
			elseif (emValues[lookUp] > emValues[emTemp])
				selectedIndex = i
				emTemp = lookUp
			endif
		endif

		i = i + 1
	endwhile
	
	;;Debug.MessageBox("Chosen threshold index = " + selectedIndex)
	return EncodeSelection(selectedIndex)
EndFunction

;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; Function: EncodeSelection
;; ---------------------------------------------------------------------------------------------------------------------------------------------
;; This function takes in a threshold index and returns the emotion code associated with it.
;; ---------------------------------------------------------------------------------------------------------------------------------------------
int Function EncodeSelection(int select)
	int code = -1
	
	if (thresholdsNames[select] == "joy")
		code = 0
	elseif (thresholdsNames[select] == "sadness")
		code = 1
	elseif (thresholdsNames[select] == "trust")
		code = 2
	elseif (thresholdsNames[select] == "disgust")
		code = 3
	elseif (thresholdsNames[select] == "anger")
		code = 4
	elseif (thresholdsNames[select] == "fear")
		code = 5
	elseif (thresholdsNames[select] == "anticipation")
		code = 6
	elseif (thresholdsNames[select] == "surprise")
		code = 7
	endif

	return code
EndFunction